@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.40';
+our $VERSION = '0.41';
 
 use Carp;
 
@@ -173,16 +173,18 @@ For example, to expire an accepted connection after 30 seconds of inactivity:
  on_accept => sub {
     my ( $newclient ) = @_;
 
-    my $stream;
-
     my $watchdog = IO::Async::Timer::Countdown->new(
        delay => 30,
 
-       on_expire => sub { $stream->close },
-    );
-    $stream->add_child( $watchdog );
+       on_expire => sub {
+          my $self = shift;
 
-    $stream = IO::Async::Stream->new(
+          my $stream = $self->parent;
+          $stream->close;
+       },
+    );
+
+    my $stream = IO::Async::Stream->new(
        handle => $newclient,
 
        on_read => sub {
@@ -197,11 +199,48 @@ For example, to expire an accepted connection after 30 seconds of inactivity:
        },
     ) );
 
+    $stream->add_child( $watchdog );
     $watchdog->start;
 
     $loop->add( $watchdog );
  }
 
+Rather than setting up a lexical variable to store the Stream so that the
+Timer's C<on_expire> closure can call C<close> on it, the parent/child
+relationship between the two Notifier objects is used. At the time the Timer
+C<on_expire> closure is invoked, it will have been added as a child notifier
+of the Stream; this means the Timer's C<parent> method will return the Stream
+Notifier. This enables it to call C<close> without needing to capture a
+lexical variable, which would create a cyclic reference.
+
+=head2 Fixed-Delay Repeating Timer
+
+The C<on_expire> event fires a fixed delay after the C<start> method has begun
+the countdown. The C<start> method can be invoked again at some point during
+the C<on_expire> handling code, to create a timer that invokes its code
+regularly a fixed delay after the previous invocation has finished. This
+creates an arrangement similar to an L<IO::Async::Timer::Periodic>, except
+that it will wait until the previous invocation has indicated it is finished,
+before starting the countdown for the next call.
+
+ my $timer = IO::Async::Timer::Countdown->new(
+    delay => 60,
+
+    on_expire => sub {
+       my $self = shift;
+
+       start_some_operation(
+          on_complete => sub { $self->start },
+       );
+    },
+ );
+
+ $timer->start;
+ $loop->add( $timer );
+
+This example invokes the C<start_some_operation> function 60 seconds after the
+previous iteration has indicated it has finished.
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
