@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.40';
+our $VERSION = '0.41';
 
 use Carp;
 
@@ -46,7 +46,10 @@ to execute, but runs at regular intervals beginning at the time the timer was
 started, then adding each interval thereafter.
 
 For a C<Timer> object that only runs a callback once, after a given delay, see
-instead L<IO::Async::Timer::Countdown>.
+instead L<IO::Async::Timer::Countdown>. A Countdown timer can also be used to
+create repeating events that fire at a fixed delay after the previous event
+has finished processing. See als the examples in
+C<IO::Async::Timer::Countdown>.
 
 =cut
 
@@ -76,6 +79,17 @@ CODE reference for the C<on_tick> event.
 The interval in seconds between invocations of the callback or method. Cannot
 be changed if the timer is running.
 
+=item first_interval => NUM
+
+Optional. If defined, the interval in seconds after calling the C<start>
+method before the first invocation of the callback or method. Thereafter, the
+regular C<interval> will be used. If not supplied, the first interval will be
+the same as the others.
+
+Even if this value is zero, the first invocation will be made asynchronously,
+by the containing C<Loop> object, and not synchronously by the C<start> method
+itself.
+
 =back
 
 Once constructed, the timer object will need to be added to the C<Loop> before
@@ -105,6 +119,15 @@ sub configure
       $self->{interval} = $interval;
    }
 
+   if( exists $params{first_interval} ) {
+      $self->is_running and croak "Cannot configure 'first_interval' of a running timer\n";
+
+      my $first_interval = delete $params{first_interval};
+      $first_interval >= 0 or croak "Expected a 'first_interval' as a non-negative number";
+
+      $self->{first_interval} = $first_interval;
+   }
+
    unless( $self->can_event( 'on_tick' ) ) {
       croak 'Expected either a on_tick callback or an ->on_tick method';
    }
@@ -112,6 +135,13 @@ sub configure
    $self->SUPER::configure( %params );
 }
 
+sub _next_interval
+{
+   my $self = shift;
+   return $self->{first_interval} if defined $self->{first_interval};
+   return $self->{interval};
+}
+
 sub start
 {
    my $self = shift;
@@ -121,10 +151,10 @@ sub start
    # the Loop
    if( my $loop = $self->get_loop ) {
       if( !defined $self->{next_time} ) {
-         $self->{next_time} = $loop->time + $self->{interval};
+         $self->{next_time} = $loop->time + $self->_next_interval;
       }
       else {
-         $self->{next_time} += $self->{interval};
+         $self->{next_time} += $self->_next_interval;
       }
    }
 
@@ -146,6 +176,8 @@ sub _make_cb
    return $self->_capture_weakself( sub {
       my $self = shift;
 
+      undef $self->{first_interval};
+
       undef $self->{id};
       $self->start;
 
